"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Entity = exports.toPlainDynamicIdentifier = exports.toDynamicIdentifier = exports.isEntity = void 0;
const decorator_error_1 = require("../errors/decorator.error");
const policy_1 = require("../interfaces/policy");
const constants_1 = require("./constants");
function isEntity(target) {
    return (target &&
        typeof target === 'object' &&
        target.constructor &&
        typeof target.constructor === 'function' &&
        Reflect.hasMetadata(constants_1.ENTITY_META_FIELD, target.constructor) &&
        Reflect.hasMetadata(constants_1.ID_META_FIELD, target.constructor) &&
        typeof Reflect.getMetadata(constants_1.ENTITY_META_FIELD, target.constructor) ===
            'function' &&
        typeof Reflect.getMetadata(constants_1.ID_META_FIELD, target.constructor) === 'function');
}
exports.isEntity = isEntity;
function toDynamicIdentifier(target) {
    if (!isEntity(target)) {
        throw new decorator_error_1.DecoratorError('Target object should use @Entity() decorator');
    }
    return {
        [policy_1.ENTITY_FIELD]: Reflect.getMetadata(constants_1.ENTITY_META_FIELD, target.constructor)(target),
        [policy_1.ID_FIELD]: Reflect.getMetadata(constants_1.ID_META_FIELD, target.constructor)(target),
    };
}
exports.toDynamicIdentifier = toDynamicIdentifier;
function toPlainDynamicIdentifier(target) {
    const dynamicIdentifier = toDynamicIdentifier(target);
    return `${dynamicIdentifier[policy_1.ENTITY_FIELD]}${policy_1.DELIMITER}${dynamicIdentifier[policy_1.ID_FIELD]}`;
}
exports.toPlainDynamicIdentifier = toPlainDynamicIdentifier;
function Entity(options) {
    return (target) => {
        if ((!options || (!options.name && !options.nameField)) && !target.name) {
            throw new decorator_error_1.DecoratorError('Entity has no explicit name, nameField nor constructor defined');
        }
        Reflect.defineMetadata(constants_1.ENTITY_META_FIELD, (instance) => options && options.nameField
            ? instance[options.nameField]
            : options && options.name
                ? options.name
                : target.name.toLowerCase(), target);
        Reflect.defineMetadata(constants_1.ID_META_FIELD, (instance) => options && options.idField
            ? instance[options.idField]
            : instance[policy_1.ID_FIELD], target);
    };
}
exports.Entity = Entity;
