"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.dynamicIdentifierExtractor = exports.processTemplate = exports.VAR_REGEXP = void 0;
const dotProp = require("dot-prop");
const entity_1 = require("./entity");
const decorator_error_1 = require("../errors/decorator.error");
exports.VAR_REGEXP = /{(?<var>[^}]+)}/gm;
function processTemplate(template, context) {
    let match;
    while ((match = exports.VAR_REGEXP.exec(template)) !== null) {
        if (match.index === exports.VAR_REGEXP.lastIndex) {
            exports.VAR_REGEXP.lastIndex++;
        }
        let [definition, varPath] = match;
        varPath = varPath.trim();
        if (!dotProp.has(context, varPath)) {
            throw new decorator_error_1.DecoratorError(`Unable to find ${varPath} property in Request from metadata field`);
        }
        let varValue = dotProp.get(context, varPath);
        if (entity_1.isEntity(varValue)) {
            varValue = entity_1.toPlainDynamicIdentifier(varValue);
        }
        template = template.replace(definition, varValue.toString());
    }
    return template;
}
exports.processTemplate = processTemplate;
function dynamicIdentifierExtractor(metadataField) {
    return (target, ctx) => {
        if (!Reflect.hasMetadata(metadataField, target)) {
            return null;
        }
        const value = Reflect.getMetadata(metadataField, target);
        if (!ctx || typeof value !== 'string') {
            return value;
        }
        const request = ctx.switchToHttp().getRequest();
        return processTemplate(value, request);
    };
}
exports.dynamicIdentifierExtractor = dynamicIdentifierExtractor;
