"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Firewall = void 0;
const tslib_1 = require("tslib");
const policy_1 = require("./interfaces/policy");
const matcher_1 = require("./matcher");
const memory_storage_1 = require("./storages/memory.storage");
const policy_vector_1 = require("./policy-vector");
const core_1 = require("./helpers/core");
class Firewall extends core_1.CoreHelper {
    constructor(storage, matcher) {
        super();
        this.storage = storage;
        this.matcher = matcher;
    }
    static create(storage, matcher) {
        return new this(storage || new memory_storage_1.MemoryStorage(), matcher || new matcher_1.Matcher());
    }
    isAllowed(rawResource, rawAction, rawPrincipal) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const resolution = yield this.resolve(rawResource, rawAction, rawPrincipal);
            return resolution.allow.length > 0 && resolution.deny.length <= 0;
        });
    }
    isAllowedAny(rawResource, rawAction, rawPrincipal) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const resolution = yield this.resolve(rawResource, rawAction, rawPrincipal);
            return resolution.allow.length > 0;
        });
    }
    isAllowedImplicit(rawResource, rawAction, rawPrincipal) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const resolution = yield this.resolve(rawResource, rawAction, rawPrincipal);
            return resolution.deny.length <= 0;
        });
    }
    resolve(rawResource, rawAction, rawPrincipal) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const principal = (this.normalizeDynamicIdentifier(rawPrincipal, policy_1.PRINCIPAL));
            const rawPolicies = yield this.storage.fetch(principal);
            const policies = policy_vector_1.PolicyVector.create(...rawPolicies);
            return this.matcher.match(rawResource, rawAction, principal, policies);
        });
    }
}
exports.Firewall = Firewall;
