"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Policy = void 0;
const clone = require("clone");
const policy_1 = require("./interfaces/policy");
const core_1 = require("./helpers/core");
const missing_policy_props_error_1 = require("./errors/missing-policy-props.error");
class Policy extends core_1.CoreHelper {
    constructor(rawPolicy) {
        super();
        let policy = {};
        if (typeof rawPolicy === 'string') {
            policy = JSON.parse(rawPolicy);
        }
        else {
            policy = clone(rawPolicy);
        }
        this.normalizePolicyDynamicIdentifiers(policy);
        missing_policy_props_error_1.MissingPolicyProps.assert(policy);
        Object.assign(this, policy);
    }
    property(prop) {
        if (policy_1.DYNAMIC_IDENTIFIER_PROPS.includes(prop)) {
            if (!this.isDynamicVector(this[prop])) {
                if (!this.isDynamicItem(this[prop])) {
                    return [];
                }
                return [this[prop].parse()];
            }
            return this[prop].map((item) => item.parse());
        }
        return this[prop];
    }
    toJSON(parse = false) {
        const { Sid, Effect, Action, Resource, Principal } = this;
        const rawPolicy = clone({ Sid, Effect, Action, Resource, Principal });
        if (parse) {
            this.unpackPolicyDynamicIdentifiers(rawPolicy);
        }
        else {
            this.removePolicyDynamicIdentifiers(rawPolicy);
        }
        return rawPolicy;
    }
    unpackPolicyDynamicIdentifiers(rawPolicy) {
        for (const prop of policy_1.DYNAMIC_IDENTIFIER_PROPS) {
            if (this.isDynamicVector(rawPolicy[prop])) {
                for (const i in rawPolicy[prop]) {
                    if (this.isDynamicItem(rawPolicy[prop][i])) {
                        rawPolicy[prop][i] = rawPolicy[prop][i].parse();
                    }
                    continue;
                }
                continue;
            }
            if (this.isDynamicItem(rawPolicy[prop])) {
                rawPolicy[prop] = rawPolicy[prop].parse();
            }
        }
    }
    removePolicyDynamicIdentifiers(rawPolicy) {
        for (const prop of policy_1.DYNAMIC_IDENTIFIER_PROPS) {
            if (this.isDynamicVector(rawPolicy[prop])) {
                for (const i in rawPolicy[prop]) {
                    if (this.isDynamicItem(rawPolicy[prop][i])) {
                        rawPolicy[prop][i] = rawPolicy[prop][i].value;
                    }
                    continue;
                }
                continue;
            }
            if (this.isDynamicItem(rawPolicy[prop])) {
                rawPolicy[prop] = rawPolicy[prop].value;
            }
        }
    }
    normalizePolicyDynamicIdentifiers(rawPolicy) {
        const self = this;
        for (const prop of policy_1.DYNAMIC_IDENTIFIER_PROPS) {
            if (!rawPolicy.hasOwnProperty(prop)) {
                continue;
            }
            if (this.isDynamicVector(rawPolicy[prop])) {
                for (const i in rawPolicy[prop]) {
                    if (this.isDynamicItem(rawPolicy[prop][i])) {
                        continue;
                    }
                    rawPolicy[prop][i] = {
                        value: rawPolicy[prop][i].toString(),
                        parse() {
                            return self.parseDynamicIdentifier(rawPolicy, prop, this.value);
                        },
                    };
                }
                continue;
            }
            if (this.isDynamicItem(rawPolicy[prop])) {
                continue;
            }
            rawPolicy[prop] = {
                value: rawPolicy[prop].toString(),
                parse() {
                    return self.parseDynamicIdentifier(rawPolicy, prop, this.value);
                },
            };
        }
    }
}
exports.Policy = Policy;
