"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CachedStorage = void 0;
const tslib_1 = require("tslib");
const cache_error_1 = require("./cache/cache.error");
class CachedStorage {
    constructor(storage, cache, options) {
        this.storage = storage;
        this.cache = cache;
        this.options = options;
        this.readonly = false;
        this.DEFAULT_OPTIONS = {
            expire: 3600,
            prefix: 'IACRY_PCACHE/',
        };
        if (!options) {
            this.options = this.DEFAULT_OPTIONS;
        }
        this.options = !options
            ? this.DEFAULT_OPTIONS
            : Object.assign({}, this.DEFAULT_OPTIONS, this.options);
    }
    fetch(principal) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const key = this.key(principal);
            if (yield this.cache.has(key)) {
                return this.decode(yield this.cache.get(key));
            }
            const policies = yield this.storage.fetch(principal);
            if (!(yield this.cache.set(key, this.encode(policies), this.options.expire))) {
                throw new cache_error_1.CacheError(`Unable to store policy cache for key: ${key}`);
            }
            return policies;
        });
    }
    save(principal, rawPolicies) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.purgeCache(principal);
            return this.storage.save(principal, rawPolicies);
        });
    }
    add(principal, rawPolicies) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.purgeCache(principal);
            return this.storage.add(principal, rawPolicies);
        });
    }
    purge(principal) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.purgeCache(principal);
            return this.storage.purge(principal);
        });
    }
    purgeCache(principal) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const key = this.key(principal);
            if (yield this.cache.has(key)) {
                if (!(yield this.cache.remove(key))) {
                    throw new cache_error_1.CacheError(`Unable to remove policy cache for key: ${key}`);
                }
            }
        });
    }
    encode(policies) {
        return JSON.stringify(policies);
    }
    decode(rawPolicies) {
        return JSON.parse(rawPolicies);
    }
    key(principal) {
        return `${this.options.prefix}${principal.entity}/${principal.id}`;
    }
}
exports.CachedStorage = CachedStorage;
