"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MultipleStorage = void 0;
const tslib_1 = require("tslib");
class MultipleStorage {
    constructor(storages = []) {
        this.storages = storages;
        this.readonly = false;
    }
    fetch(principal) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const result = yield Promise.all(this.storages.map((storage) => storage.fetch(principal)));
            return result.reduce((accumulator, current) => {
                return accumulator.concat(...current);
            });
        });
    }
    save(principal, rawPolicies) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const result = yield Promise.all(this.storages
                .filter((storage) => !storage.readonly)
                .map((storage) => storage.save(principal, rawPolicies)));
            return result.reduce((accumulator, current) => {
                return Math.min(accumulator, current);
            });
        });
    }
    add(principal, rawPolicies) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const result = yield Promise.all(this.storages
                .filter((storage) => !storage.readonly)
                .map((storage) => storage.add(principal, rawPolicies)));
            return result.reduce((accumulator, current) => {
                return Math.min(accumulator, current);
            });
        });
    }
    purge(principal) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const result = yield Promise.all(this.storages
                .filter((storage) => !storage.readonly)
                .map((storage) => storage.purge(principal)));
            return result.reduce((accumulator, current) => {
                return Math.max(accumulator, current);
            });
        });
    }
}
exports.MultipleStorage = MultipleStorage;
