"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PoliciesStorage = void 0;
const tslib_1 = require("tslib");
const sequelize_typescript_1 = require("sequelize-typescript");
const sequelize_1 = require("sequelize");
const core_1 = require("../../helpers/core");
let PoliciesStorage = class PoliciesStorage extends sequelize_typescript_1.Model {
    static savePrincipalPolicies(principal, rawPolicies, attach = true) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!attach) {
                yield this.destroyByPrincipal(principal);
            }
            const result = yield this.bulkCreate(rawPolicies.map((policy) => ({
                entity: principal.entity === core_1.CoreHelper.ANY
                    ? null
                    : this.normalizePolicyItem(principal.entity),
                id: principal.id === core_1.CoreHelper.ANY ? null : principal.id,
                policy: typeof policy === 'string' ? policy : JSON.stringify(policy),
            })));
            return result.length;
        });
    }
    static destroyByPrincipal(principal) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const payload = {
                where: {
                    entity: {
                        [sequelize_1.Op.or]: [null],
                    },
                    id: {
                        [sequelize_1.Op.or]: [null],
                    },
                },
            };
            if (principal.entity === core_1.CoreHelper.ANY) {
                delete payload.where.entity;
            }
            else {
                payload.where.entity[sequelize_1.Op.or].push(this.normalizePolicyItem(principal.entity));
            }
            if (principal.id === core_1.CoreHelper.ANY) {
                delete payload.where.id;
            }
            else {
                payload.where.id[sequelize_1.Op.or].push(principal.id);
            }
            if (!payload.where.entity && !payload.where.id) {
                delete payload.where;
            }
            return this.destroy(payload);
        });
    }
    static findByPrincipal(principal) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const payload = {
                where: {
                    entity: {
                        [sequelize_1.Op.or]: [null],
                    },
                    id: {
                        [sequelize_1.Op.or]: [null],
                    },
                },
                attributes: ['policy'],
            };
            if (principal.entity === core_1.CoreHelper.ANY) {
                delete payload.where.entity;
            }
            else {
                payload.where.entity[sequelize_1.Op.or].push(this.normalizePolicyItem(principal.entity));
            }
            if (principal.id === core_1.CoreHelper.ANY) {
                delete payload.where.id;
            }
            else {
                payload.where.id[sequelize_1.Op.or].push(principal.id);
            }
            if (!payload.where.entity && !payload.where.id) {
                delete payload.where;
            }
            const entries = yield this.findAll(payload);
            return entries.map(({ policy }) => policy);
        });
    }
    static normalizePolicyItem(item) {
        return item.toLowerCase();
    }
};
tslib_1.__decorate([
    sequelize_typescript_1.PrimaryKey,
    sequelize_typescript_1.AutoIncrement,
    sequelize_typescript_1.Column,
    tslib_1.__metadata("design:type", Number)
], PoliciesStorage.prototype, "pk", void 0);
tslib_1.__decorate([
    sequelize_typescript_1.AllowNull(true),
    sequelize_typescript_1.Column,
    tslib_1.__metadata("design:type", String)
], PoliciesStorage.prototype, "id", void 0);
tslib_1.__decorate([
    sequelize_typescript_1.AllowNull(true),
    sequelize_typescript_1.Column,
    tslib_1.__metadata("design:type", String)
], PoliciesStorage.prototype, "entity", void 0);
tslib_1.__decorate([
    sequelize_typescript_1.Column(sequelize_typescript_1.DataType.TEXT),
    tslib_1.__metadata("design:type", String)
], PoliciesStorage.prototype, "policy", void 0);
tslib_1.__decorate([
    sequelize_typescript_1.CreatedAt,
    tslib_1.__metadata("design:type", Date)
], PoliciesStorage.prototype, "createdAt", void 0);
tslib_1.__decorate([
    sequelize_typescript_1.UpdatedAt,
    tslib_1.__metadata("design:type", Date)
], PoliciesStorage.prototype, "updatedAt", void 0);
tslib_1.__decorate([
    sequelize_typescript_1.DeletedAt,
    tslib_1.__metadata("design:type", Date)
], PoliciesStorage.prototype, "deletedAt", void 0);
PoliciesStorage = tslib_1.__decorate([
    sequelize_typescript_1.Table({
        timestamps: true,
        paranoid: true,
    })
], PoliciesStorage);
exports.PoliciesStorage = PoliciesStorage;
